/**
 * Copyright (C) 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package casino.ebean;

import java.util.List;

import models.casino.User;

import org.slf4j.Logger;

import casino.CasinoUserManager;

import com.avaje.ebean.EbeanServer;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class EbeanUserManager implements CasinoUserManager {

    private final EbeanServer ebeanServer;
    private final Logger logger;

    @Inject
    public EbeanUserManager(Logger logger, EbeanServer ebeanServer) {
        this.logger = logger;
        this.ebeanServer = ebeanServer;

    }

    @Override
    public boolean createNewCasinoUser(String email,
                                       String passwordHash,
                                       String confirmationCode) {

        User user = getOneUserWithEmail(email);

        if (user != null) {

            return false;
        }

        user = new User(email, passwordHash, confirmationCode);
        ebeanServer.save(user);

        return true;

    }

    @Override
    public boolean isUserActivated(String email) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return false;
        }

        // make sure the user confirmed the name
        if (user.getConfirmationCode().length() != 0) {
            return false;
        }

        return true;
    }

    @Override
    public boolean hasRole(String email, String role) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return false;
        }

        return user.hasRole(role);

    }

    @Override
    public void addRole(String email, String role) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return;
        }

        user.addRole(role);
        ebeanServer.save(user);

    }

    @Override
    public void removeRole(String email, String role) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return;
        }

        user.removeRole(role);
        ebeanServer.save(user);

    }

    @Override
    public void setRecoveryPasswordCode(String email,
                                        String recoveryPasswordCode) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return;
        }

        user.setRecoverPasswordCode(recoveryPasswordCode);
        ebeanServer.save(user);

    }

    @Override
    public String getCasinoUserWithConfirmationCode(String confirmationCode) {

        List<User> users = ebeanServer.find(User.class).where()
                .eq("confirmationCode", confirmationCode).findList();

        if (users.size() == 0) {

            return null;

        } else if (users.size() > 1) {

            logger.error("more than one User with confirmation code "
                    + confirmationCode
                    + " (Ebean) found. That's an inconsistency - taking the first one...");

        }

        return users.get(0).getEmail();

    }

    @Override
    public void deleteConfirmationCodeOfCasioUser(String email) {

        System.out.println("try delete of confiurmation code:");
        
        User user = getOneUserWithEmail(email);

        if (user == null) {
            System.out.println("user is null...");
            return;
        }

        System.out.println("confirmation code is: " + user.getConfirmationCode());
        user.setConfirmationCode(null);
        
        System.out.println("saving bean!");
        ebeanServer.save(user);

    }

    @Override
    public String getUserPasswordHash(String email) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return null;
        }

        return user.getPwHash();
    }

    @Override
    public void setNewPasswordHashForUser(String email, String passwordHash) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return;
        }

        user.setPwHash(passwordHash);

        ebeanServer.save(user);

    }

    @Override
    public boolean doesUserExist(String email) {

        User user = getOneUserWithEmail(email);

        if (user == null) {
            return false;
        }

        return true;
    }

    @Override
    public String getCasinoUserWithRecoveryPasswordCode(String recoverPasswordCode) {

        List<User> users = ebeanServer.find(User.class).where()
                .eq("recoverPasswordCode", recoverPasswordCode).findList();

        if (users.size() == 0) {

            return null;

        } else if (users.size() > 1) {

            logger.error("more than one User with recoveryCode "
                    + recoverPasswordCode
                    + " (Ebean) found. That's an inconsistency - taking the first one...");

        }

        return users.get(0).getEmail();
    }

    private User getOneUserWithEmail(String email) {

        
        System.out.println("trying to fetch: " + email);
        
        
        List<User> users = ebeanServer.find(User.class).where()
                .eq("email", email).findList();

        if (users.size() == 0) {

            System.out.println("null... no entries with email: " + email);
            return null;

        } else if (users.size() > 1) {

            logger.error("more than one User "
                    + email
                    + " (Ebean) found. That's an inconsistency - taking the first one...");

        }

        return users.get(0);

    }

}
