/**
 * Copyright (C) 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package casino;

import java.util.UUID;

import ninja.utils.NinjaProperties;

import org.apache.commons.codec.binary.Base64;
import org.mindrot.jbcrypt.BCrypt;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class Casino {

    /**
     * Optional configuration property you can use to set salt factor (default
     * is 10)
     */
    private final String CASINO_BCYPT_SALT_FACTOR = "casino.bcrypt_salt_factor";

    private final int CASINO_BCYPT_SALT_FACTOR_DEFAULT = 10;

    /** This is the implementation that can write users to e.g. a database */
    private CasinoUserManager casinoUserManager;

    /** The salt factor to use when hashing passwords */
    private final int saltFactor;

    private final AfterUserCreationHook afterUserCreationHook;

    @Inject
    public Casino(AfterUserCreationHook afterUserCreationHook,
                  CasinoUserManager casinoUserManager,
                  NinjaProperties ninjaProperties) {

        this.afterUserCreationHook = afterUserCreationHook;
        this.casinoUserManager = casinoUserManager;

        saltFactor = ninjaProperties.getIntegerWithDefault(
                CASINO_BCYPT_SALT_FACTOR, CASINO_BCYPT_SALT_FACTOR_DEFAULT);

    }

    public String getHashForPassword(String password) {

        return BCrypt.hashpw(password, BCrypt.gensalt(saltFactor));

    }

    public boolean doPasswordAndHashMatch(String password, String passwordHash) {

        return BCrypt.checkpw(password, passwordHash);

    }

    /**
     * called right after a user is created...
     */
    public void executeAfterUserCreationHook(String email) {

        afterUserCreationHook.execute(email);

    }

    // ////////////////////////////////////////////////////////////////////////
    // Additional stuff:
    // ////////////////////////////////////////////////////////////////////////
    private static byte[] asByteArray(UUID uuid) {
        long msb = uuid.getMostSignificantBits();
        long lsb = uuid.getLeastSignificantBits();
        byte[] buffer = new byte[16];

        for (int i = 0; i < 8; i++) {
            buffer[i] = (byte) (msb >>> 8 * (7 - i));
        }
        for (int i = 8; i < 16; i++) {
            buffer[i] = (byte) (lsb >>> 8 * (7 - i));
        }
        return buffer;
    }

    public String shortUUID() {
        UUID uuid = UUID.randomUUID();
        return Base64.encodeBase64URLSafeString(asByteArray(uuid));
    }

    // /////////////////////////////////////////////////////////////////////////
    // convenience methods.
    // create new user also calls hook if there is one
    // /////////////////////////////////////////////////////////////////////////
    public boolean createNewCasinoUser(String email,
                                       String passwordHash,
                                       String confirmationCode) {

        boolean userCreated = casinoUserManager.createNewCasinoUser(email,
                passwordHash, confirmationCode);

        // only exec if user has been created...
        if (userCreated) {
            // and execute hook...
            executeAfterUserCreationHook(email);
        }

        return userCreated;

    }

    public boolean isUserActivated(String email) {

        return casinoUserManager.isUserActivated(email);
    }

    public boolean doesUserExist(String email) {

        return casinoUserManager.doesUserExist(email);

    }

    public String getUserPasswordHash(String email) {

        return casinoUserManager.getUserPasswordHash(email);

    }

    public void setNewPasswordHashForUser(String email, String passwordHash) {

        casinoUserManager.setNewPasswordHashForUser(email, passwordHash);

    }

    public boolean hasRole(String email, String role) {

        return casinoUserManager.hasRole(email, role);

    }

    public void addRole(String email, String role) {

        casinoUserManager.addRole(email, role);

    }

    public void removeRole(String email, String role) {

        casinoUserManager.removeRole(email, role);

    }

    public String getCasinoUserWithRecoveryPasswordCode(String recoverPasswordCode) {

        return casinoUserManager
                .getCasinoUserWithRecoveryPasswordCode(recoverPasswordCode);
    }

    public void setRecoveryPasswordCode(String email,
                                        String recoveryPasswordCode) {

        casinoUserManager.setRecoveryPasswordCode(email, recoveryPasswordCode);

    }

    public String getCasinoUserWithConfirmationCode(String confirmationCode) {

        return casinoUserManager
                .getCasinoUserWithConfirmationCode(confirmationCode);
    }

    public void deleteConfirmationCodeOfCasioUser(String email) {

        casinoUserManager.deleteConfirmationCodeOfCasioUser(email);

    }

}
