/**
 * Copyright (C) 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package controllers.casino;

import java.util.UUID;

import ninja.Context;
import ninja.Result;
import ninja.Results;
import ninja.params.Param;
import ninja.params.PathParam;
import ninja.validation.Required;
import ninja.validation.Validation;
import casino.Casino;
import casino.RegistrationMailer;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class RegistrationController {

    private final String VIEW_REGISTRATION = "views/casino/RegistrationController/registration.ftl.html";
    private final String VIEW_REGISTRATION_PENDING = "views/casino/RegistrationController/registrationPending.ftl.html";

    private final Casino casino;
    private final RegistrationMailer registrationMailer;

    @Inject
    public RegistrationController(Casino casino,
                                  RegistrationMailer registrationMailer) {
        this.casino = casino;
        this.registrationMailer = registrationMailer;
    }

    // ////////////////////////////////////////////////////////////////////////
    // Sign up (register) new user:
    // ////////////////////////////////////////////////////////////////////////
    public Result registration(Context context) {
        return Results.html();
    }

    //
    /**
     * Do the validation of the registration screen
     */
    public Result registrationFinish(@Required @Param("email") String email,
                                     @Required @Param("password") String password,
                                     @Required @Param("confirm") String confirm,
                                     @Required @Param("acceptTermsOfService") Boolean acceptTermsOfService,
                                     Validation validation,
                                     Context context) throws Exception {

        boolean hasErrors = false;

        // check if user exists and don't allow to register user:
        if (!hasErrors) {

            if (casino.doesUserExist(email)) {
                hasErrors = true;

            }
        }
        //
        // save or display error
        if (hasErrors) {

            context.getFlashCookie().error("i18nRegistrationError");

            // If acceptTermsOfService is not checked it is not sent to server..
            // therefore we check for null...
            if (acceptTermsOfService != null) {
                // params.flash("acceptTermsOfService");
            }

            return Results.html().template(VIEW_REGISTRATION);

        } else {

            String passwordHash = casino.getHashForPassword(password);
            String confirmationCode = casino.shortUUID();

            casino.createNewCasinoUser(email, passwordHash, confirmationCode);

            registrationMailer.confirmation(email, confirmationCode);

            return Results.html().template(VIEW_REGISTRATION_PENDING);

        }

    }

    /**
     * Registration was successful => show message to user that an email was
     * sent for final activation of user account.
     */
    public Result registrationPending() {

        return Results.html();
    }

    /**
     * Checks and activates user account with code sent in email.
     */
    public Result registrationConfirm(@PathParam("code") String code,
                                      Context context) {

        if (code == null || code.length() == 0) {
            context.getFlashCookie().error("i18nRegistrationPendingError");
        }

        // get user with this email

        String email = casino.getCasinoUserWithConfirmationCode(code);

        if (email != null) {
            casino.deleteConfirmationCodeOfCasioUser(email);

            // we also log in user after successful confirmation of email...
            context.getSessionCookie().put("username", email);

            context.getFlashCookie().success("i18nRegistrationPendingSuccess");
        } else {
            context.getFlashCookie().error("i18nRegistrationPendingError");
        }

        return Results.html();
    }

    // ////////////////////////////////////////////////////////////////////////
    // Recover lost password of user using email:
    // ////////////////////////////////////////////////////////////////////////

    /**
     * Show input window for email.
     */
    public Result lostPassword(Context context) {

        return Results.html();

    }
    //
    // /**
    // * Validate the input of email... check recaptcha and so on...
    // */
    // public static void lostPasswordFinish(@Required @Email String email,
    // @Recaptcha String captcha) {
    //
    // // /check:
    // checkAuthenticity();
    //
    // boolean hasErrors = true;
    //
    // if (Play.id.equals("test")) {
    //
    // // we are in test mode.. ignore wrong captcha
    // if (validation.errors().size() == 1) {
    //
    // if (validation.errors().get(0).getKey().equals("captcha")) {
    // hasErrors = false;
    // }
    // }
    //
    // } else {
    //
    // hasErrors = validation.hasErrors();
    //
    // }
    //
    // // save or display error
    // if (hasErrors) {
    //
    // flash.error("registration.error");
    // params.flash("email");
    // validation.keep();
    // lostPassword();
    //
    // } else {
    //
    // // check if user exists and send email
    // // always display success message...
    //
    // if (Casino.doesUserExist(email)) {
    // // user found.. displaying notification.
    // // send notification
    //
    // String recoverPasswordCode = Casino.shortUUID();
    //
    // Casino.setRecoveryPasswordCode(email, recoverPasswordCode);
    //
    // RegistrationMailer.lostPassword(email, recoverPasswordCode);
    //
    // // display notification to user
    // lostPasswordEmailSentCheckInbox();
    //
    // } else {
    // //
    // System.out.println("user not found.. displaying notification anyway.");
    // // display success anyway don't allow robots to check which
    // // emails are signed up.
    // lostPasswordEmailSentCheckInbox();
    //
    // }
    //
    // }
    //
    // }
    //
    // /**
    // * email lookup was successful => tell user to check inbox to re-enable
    // * password.
    // */
    // public static void lostPasswordEmailSentCheckInbox() {
    //
    // render();
    //
    // }
    //
    // /**
    // * User has valid confirmation => we allow to generate new password
    // */
    // public static void lostPasswordNewPassword(@Required String code) {
    //
    // render(code);
    //
    // }
    //
    // /**
    // * Validation for the lostPasswordNewPassword screen
    // */
    // public static void lostPasswordNewPasswordFinish(@Required String code,
    // @Recaptcha String captcha,
    // @Required @Equals("passwordConfirm") @MinSize(8) String password,
    // @Required @MinSize(8) String passwordConfirm) {
    //
    // // check:
    // checkAuthenticity();
    //
    // boolean hasErrors = true;
    //
    // if (Play.id.equals("test")) {
    //
    // // we are in test mode.. ignore wrong captcha
    // if (validation.errors().size() == 1) {
    //
    // if (validation.errors().get(0).getKey().equals("captcha")) {
    // hasErrors = false;
    // }
    // }
    //
    // } else {
    //
    // hasErrors = validation.hasErrors();
    //
    // }
    //
    // // save or display error
    // if (hasErrors) {
    //
    // flash.error("registration.error");
    // params.flash("password");
    // params.flash("passwordConfirm");
    // validation.keep();
    // lostPasswordNewPassword(code);
    //
    // } else {
    //
    // String email = Casino
    // .getCasinoUserWithRecoveryPasswordCode(code);
    //
    // if (email == null) {
    // // hmm. does not exist. display success anyway...
    // // we don't want to let robots sniff...
    //
    // render();
    //
    //
    // } else {
    //
    // Casino.setRecoveryPasswordCode(email, code);
    //
    // // compute hash...
    // String passwordHash = Casino.getHashForPassword(password);
    //
    // Casino.setNewPasswordHashForUser(email,
    // passwordHash);
    //
    // render();
    //
    // }
    //
    // }
    //
    // }

}
